#reading the config file
from panda3d.core import loadPrcFileData
from direct.showbase.AppRunnerGlobal import appRunner
if appRunner: #run from binary
    path=appRunner.p3dFilename.getDirname()+'/config.txt'    
else:  #run from python 
    path='config.txt'
try:    
    f = open(path, 'r')
    for line in f:        
        if not line.startswith('#'):        
            loadPrcFileData('',line)
except IOError:
    print "No config file, using default" 
#window size, needs to be set now or filters will go bust!    
from panda3d.core import WindowProperties
from panda3d.core import ConfigVariableInt
from panda3d.core import ConfigVariableBool
from panda3d.core import ConfigVariableString
config_aa=ConfigVariableInt('antialias', 0)
config_fulscreen=ConfigVariableBool('fullscreen')
config_win_size=ConfigVariableInt('win-size', '640 480') 
config_autoshader=ConfigVariableBool('use-shaders', 0)
config_flare=ConfigVariableBool('lens-flare', 0)
config_bloom=ConfigVariableBool('bloom', 0)
config_music=ConfigVariableInt('music-volume', '100') 
config_sfx=ConfigVariableInt('sound-volume', '100') 
config_difficulty=ConfigVariableInt('difficulty', '10') 
#keymapping
config_key_up=ConfigVariableString('up', 'w')
config_key_down=ConfigVariableString('down', 's')
config_key_right=ConfigVariableString('right', 'd')
config_key_left=ConfigVariableString('left', 'a')
config_key_thruster=ConfigVariableString('thruster', 'mouse1')
config_key_brake=ConfigVariableString('brake', 'mouse2')
config_key_zoomin=ConfigVariableString('zoomin', 'wheel_up')
config_key_zoomout=ConfigVariableString('zoomout', 'wheel_down')
config_key_lock_camera=ConfigVariableString('camera', 'mouse2')

if(config_aa.getValue()>0): 
    loadPrcFileData('','multisamples '+str(config_aa.getValue()+1)) 
wp = WindowProperties.getDefault() 
wp.setFixedSize(True)  
#wp.setUndecorated(True)  
wp.setTitle("2D Moon! -Wezu@pyWeek")       
if(config_fulscreen.getValue ()==1): 
    wp.setFullscreen(True)
wp.setSize(config_win_size.getWord(0),config_win_size.getWord(1))
WindowProperties.setDefault(wp)

#import all the rest
from panda3d.core import *
from direct.gui.DirectGui import *
from direct.showbase.DirectObject import DirectObject
import direct.directbase.DirectStart
from direct.filter.FilterManager import FilterManager
from direct.showbase.PythonUtil import fitSrcAngle2Dest
from direct.interval.IntervalGlobal import *
from direct.directutil import Mopath
from direct.interval.MopathInterval import *
from math import cos
from math import radians

class World(DirectObject):

    def __init__(self): 
        #CONFIG    
        if config_autoshader.getValue(): 
            render.setShaderAuto()
        if config_aa.getValue()==1:       
            render.setAntialias(AntialiasAttrib.MMultisample)
        if config_flare.getValue(): 
            self.setLenseFlare(Vec3(0.98, 0.98, 0.98))            
        if config_bloom.getValue():
            self.bloomBuffer=base.win.makeTextureBuffer("bloom", 512, 512)
            self.bloomBuffer.setSort(-3)
            self.bloomBuffer.setClearColor(Vec4(0,0,0,1))
            self.bloomCamera=base.makeCamera(self.bloomBuffer, lens=base.cam.node().getLens())
            glowShader=loader.loadShader("glowShader.sha")
            tempnode = NodePath(PandaNode("temp node"))
            tempnode.setShader(glowShader)
            self.bloomCamera.node().setInitialState(tempnode.getState())
            self.blurBuffer=self.makeFilterBuffer(self.bloomBuffer,  "Blur X", -2, "blur.sha")            
            self.finalcard = self.blurBuffer.getTextureCard()            
            self.finalcard.reparentTo(render2d)
            self.finalcard.setAttrib(ColorBlendAttrib.make(ColorBlendAttrib.MAdd))
        if config_music.getValue()!=0.0:
            volume=config_music.getValue()*0.01
            base.musicManager.setVolume(volume)
        if config_sfx.getValue()!=0.0:
            volume=config_sfx.getValue()*0.01
            base.sfxManagerList[0].setVolume(volume)
        
        #GUI
        self.font = loader.loadFont('clock_font.ttf')
        self.font.setPixelsPerUnit(45)
        self.timeFrame = DirectFrame(frameSize=(-128, 0, 0, 128),
                                    frameColor=(1, 1, 1, 1),
                                    frameTexture='tex/gui_clock.png',
                                    parent=pixel2d) 
        self.timeFrame.setTransparency(TransparencyAttrib.MAlpha)
        self.timeFrame.setPos(128,0,-128)
        self.timer=DirectLabel(text = '00:00',
                                frameColor=(0, 0, 0, 0),
                                text_font=self.font,                                
                                text_scale = 45,
                                text_fg=(0,1,1,1),                                
                                text_align=TextNode.ALeft, 
                                textMayChange=1,
                                parent=pixel2d
                                )
        self.timer.setPos(0,0,-50) 
        self.fuelFrame1 = DirectFrame(frameSize=(-128, 0, 0, 128),
                                    frameColor=(1, 1, 1, 1),
                                    frameTexture='tex/gui_empty.png',
                                    parent=pixel2d) 
        self.fuelFrame1.setTransparency(TransparencyAttrib.MAlpha)
        self.fuelFrame1.setPos(128,0,-base.win.getProperties().getYSize())
        self.fuelFrame2 = DirectFrame(frameSize=(-128, 0, 0, 128),
                                    frameColor=(1, 1, 1, 1),
                                    frameTexture='tex/gui_full.png',
                                    parent=pixel2d) 
        self.fuelFrame2.setTransparency(TransparencyAttrib.MAlpha)
        self.fuelFrame2.setPos(128,0,-base.win.getProperties().getYSize()) # -10 per 'tick' 12x tick
        
        self.speeder=DirectLabel(text = '0',
                                frameColor=(0, 0, 0, 0),
                                text_font=self.font,                                
                                text_scale = 45,
                                text_fg=(0,1,1,1),                                
                                text_align=TextNode.ALeft, 
                                textMayChange=1,
                                parent=pixel2d
                                )
        self.speeder.setPos(60,0,-base.win.getProperties().getYSize()+4) 
        #load spacebox
        self.spacebox = loader.loadModel('space_box')        
        self.spacebox.setScale(1000)
        self.spacebox.setBin('background', 1)
        self.spacebox.setDepthWrite(0)
        self.spacebox.setLightOff()
        self.spacebox.reparentTo(render)
        
        #load the starship and stuff...
        self.shipNonde=render.attachNewNode('shipNode')
        self.shipNonde.setY(50)
        self.ship=loader.loadModel('ship')        
        self.ship.reparentTo(render)        
        self.arrow=loader.loadModel('arrow')        
        self.arrow.reparentTo(self.shipNonde)
        self.arrow.setLightOff()
        #engine glow
        self.glow_back1=loader.loadModel('glow_blue')  
        self.glow_back1.reparentTo(self.ship)
        self.glow_back1.setLightOff()
        self.glow_back1.setPos(2.5, 3.5, -1.4)
        self.glow_back1.setScale(2)
        self.glow_back1.hide()
        self.glow_back2=loader.loadModel('glow_blue')  
        self.glow_back2.reparentTo(self.ship)
        self.glow_back2.setLightOff()
        self.glow_back2.setPos(-2.5, 3.5, -1.4)
        self.glow_back2.setScale(2)          
        self.glow_back2.hide()
        self.glow_front1=loader.loadModel('glow_red')  
        self.glow_front1.reparentTo(self.ship)
        self.glow_front1.setLightOff()
        self.glow_front1.setPos(2.5, -1.8, -1.4)
        self.glow_front1.setScale(2)
        self.glow_front1.hide()
        self.glow_front2=loader.loadModel('glow_red')  
        self.glow_front2.reparentTo(self.ship)
        self.glow_front2.setLightOff()
        self.glow_front2.setPos(-2.5, -1.8, -1.4)
        self.glow_front2.setScale(2)
        self.glow_front2.hide()
        self.pLight1 = PointLight('plight')
        self.pLight1.setColor(VBase4(1, 0, 0, 1))
        self.pLight1.setAttenuation(Point3(0, 0.0, 0.01))          
        self.frontLightNode = render.attachNewNode(self.pLight1) 
        self.frontLightNode.reparentTo(self.ship)
        self.frontLightNode.setPos(0, -2.5, -1.4)        
        render.setLight(self.frontLightNode)        
        self.pLight2 = PointLight('plight')
        self.pLight2.setColor(VBase4(0.09, 0.56, 1, 1))
        self.pLight2.setAttenuation(Point3(0, 0.0, 0.01))        
        self.backLightNode = render.attachNewNode(self.pLight2) 
        self.backLightNode.reparentTo(self.ship)
        self.backLightNode.setPos(0, 4, -1.4)        
        render.setLight(self.backLightNode)
        
        #load asteroids
        asteroid_lod = FadeLODNode('asteroid_lod')
        asteroid = NodePath(asteroid_lod)
        #asteroids.reparentTo(render)
        #lod2 = loader.loadModel("aster_l2")
        #lod2.reparentTo(asteroid)
        #asteroid_lod.addSwitch(500, 400)
        lod1 = loader.loadModel("aster1")
        lod1.reparentTo(asteroid)
        asteroid_lod.addSwitch(300, 150)
        lod2 = loader.loadModel("aster2")
        lod2.reparentTo(asteroid)
        asteroid_lod.addSwitch(150, 0)
        #asteroids.hide()
        asteroids=[]
        placer=loader.loadModel('aster_placer2.egg')
        for place in placer.findAllMatches("**/asteroids*"):
            asteroids.append(asteroid.copyTo(render))
            asteroids[-1].setPos(place.getPos(render))
            asteroids[-1].setHpr(place.getHpr(render))            
        #gates    
        gates=loader.loadModel('gates')            
        gates.reparentTo(render)        
        #guid
        guid=loader.loadModel('guid') 
        guid.reparentTo(render)
        guid.setLightOff()
        guid2=guid.copyTo(render)        
        guidPath = Mopath.Mopath()
        guidPath.loadFile("mopath")
        MopathInterval(guidPath, guid, name = "mopathInter", duration=6).loop()
        self.moveInterval=MopathInterval(guidPath, guid2, name = "mopathInter2", duration=6)
        
        
        #collisions
        self.tunel=loader.loadModel('tunel') 
        self.tunel.reparentTo(render)
        #tunel.ls()
        cs = CollisionSphere(0, 0, 0, 6.0)   
        cNode=CollisionNode('PC_coll')                
        self.collision = self.ship.attachNewNode(cNode)
        self.collision.node().addSolid(cs)
        #self.collision.show()
        base.cTrav = CollisionTraverser()
        self.CollQueue = CollisionHandlerQueue()
        base.cTrav.addCollider(self.collision, self.CollQueue)
        #base.cTrav.showCollisions(render)
        
        #load sounds
        self.sounds={'boom': base.loader.loadSfx("sfx/boom.ogg"),
                    'end': base.loader.loadSfx("sfx/end.ogg"),
                    'at_end': base.loader.loadSfx("sfx/at_end.ogg"),
                    'engine': base.loader.loadSfx("sfx/engine.ogg"),
                    'engine2': base.loader.loadSfx("sfx/engine2.ogg"),
                    'enter': base.loader.loadSfx("sfx/enter.ogg"),
                    'final_fail': base.loader.loadSfx("sfx/fail2.ogg"),
                    'fail': base.loader.loadSfx("sfx/fail.ogg"),
                    'hit1': base.loader.loadSfx("sfx/hit1.ogg"),
                    'hit2': base.loader.loadSfx("sfx/hit2.ogg"),
                    'hull': base.loader.loadSfx("sfx/hull.ogg"),
                    'ooc': base.loader.loadSfx("sfx/out_of_course.ogg"),
                    'low_fuel': base.loader.loadSfx("sfx/low_f.ogg")}
        self.sounds['engine'].setLoop(True)            
        self.sounds['engine2'].setLoop(True)         
        #load music
        self.music = base.loadMusic("sfx/evasion_matthew_pablo.ogg")
        self.music.setLoop(True) 
        self.music.play()
        
        #camera controll
        #base.camLens.setNearFar(5, 600)
        base.disableMouse()
        base.camera.setY(40)
        base.camera.setH(-180)
        self.cameraNode = NodePath(PandaNode("cameraNode"))        
        self.cameraNode.reparentTo(render)
        base.camera.reparentTo(self.cameraNode)
        
        #set keys
        self.keyMap = { "up" : False,
                        "down" : False,
                        "left" : False,
                        "right" : False,
                        "go" : False,
                        "stop" : False}
        self.accept(config_key_up.getValue(), self.keyMap.__setitem__, ["up", True])
        self.accept(config_key_up.getValue()+"-up", self.keyMap.__setitem__, ["up", False])                
        self.accept(config_key_down.getValue(), self.keyMap.__setitem__, ["down", True])
        self.accept(config_key_down.getValue()+"-up", self.keyMap.__setitem__, ["down", False])
        self.accept(config_key_left.getValue(), self.keyMap.__setitem__, ["left", True])
        self.accept(config_key_left.getValue()+"-up", self.keyMap.__setitem__, ["left", False])
        self.accept(config_key_right.getValue(), self.keyMap.__setitem__, ["right", True])
        self.accept(config_key_right.getValue()+"-up", self.keyMap.__setitem__, ["right", False])
        self.accept(config_key_thruster.getValue(), self.keyMap.__setitem__, ["go", True])
        self.accept(config_key_thruster.getValue()+"-up", self.keyMap.__setitem__, ["go", False])
        self.accept(config_key_brake.getValue(), self.keyMap.__setitem__, ["stop", True])
        self.accept(config_key_brake.getValue()+"-up", self.keyMap.__setitem__, ["stop", False])
        
        self.accept(config_key_zoomin.getValue(), self.zoom, [1])
        self.accept(config_key_zoomout.getValue(),self.zoom, [-1]) 
        
        self.accept(config_key_lock_camera.getValue(),self.togleCamera) 
        
        
        #set lights
        sun_light = DirectionalLight('sun') 
        sun_light.setColor(VBase4(0.8, 0.8, 0.5, 1))     
        self.sun = render.attachNewNode(sun_light)
        self.sun.setH(90)
        render.setLight(self.sun)
        alight = AmbientLight('alight')
        alight.setColor(VBase4(0.05, 0.05, 0.08, 1))
        self.ambient = render.attachNewNode(alight)
        render.setLight(self.ambient)
        
        #game-state variables
        self.speed=0
        self.engineOn=False
        self.lockCamera=False
        self.fuel=120.0
        self.TickTime=0
        self.fuel_usage=config_difficulty.getValue()/10
        
        #start all tasks    
        taskMgr.add(self.hideLoadscreen, 'hideLoadscreenTask')
        taskMgr.add(self.update, 'updateTask')
        taskMgr.add(self.shipRotation, 'shipRotationTask')
        taskMgr.doMethodLater(1, self.tick,'ticker')
        
    def tick(self, task):
        self.TickTime +=1
        sec=self.TickTime
        mins = sec / 60
        sec -= 60*mins
        self.timer['text']= '%02d:%02d' % (mins, sec)   
        if self.TickTime ==3:
            self.moveInterval.loop()
        if self.fuel < 40.0:
            if self.sounds['low_fuel'].status() != self.sounds['low_fuel'].PLAYING:
                    self.sounds['low_fuel'].play()
        self.fuelFrame2.setPos(128,0,-base.win.getProperties().getYSize()-120+int(round(self.fuel/10.0)*10.0))
        if self.fuel <= 0.0:                        
            self.timer['text_fg']=(1,0,0,1)
            self.arrow.hide()
            self.ignoreAll()
            self.keyMap["go"]=0
            self.keyMap["stop"]=0
            self.keyMap["left"]=0
            self.keyMap["right"]=0
            self.engineOn=False
            self.pLight2.setAttenuation(Point3(0, 0.0, 1)) 
            self.pLight1.setAttenuation(Point3(0, 0.0, 1)) 
            self.glow_back1.hide()
            self.glow_back2.hide() 
            self.glow_front1.hide()
            self.glow_front2.hide() 
            self.sounds['engine'].stop()
            self.sounds['engine2'].stop()            
            return task.done
        return task.again
    def togleCamera(self):
        if self.lockCamera:
            self.lockCamera=False
        else:
            self.lockCamera=True
        
    def zoom(self, howFar):
        base.camera.setY(base.camera, howFar*5)
        
    def hideLoadscreen(self, task):
        load_screen.hide()
        return task.done   
    
    def makeFilterBuffer(self, srcbuffer, name, sort, prog):
        blurBuffer=base.win.makeTextureBuffer(name, 512, 512)
        blurBuffer.setSort(sort)
        blurBuffer.setClearColor(Vec4(0,0,1,1))
        blurCamera=base.makeCamera2d(blurBuffer)
        blurScene=NodePath("new Scene")
        blurCamera.node().setScene(blurScene)
        shader = loader.loadShader(prog)
        card = srcbuffer.getTextureCard()
        card.reparentTo(blurScene)
        card.setShader(shader)
        return blurBuffer
        
    def setLenseFlare(self, threshold = Vec3(0.5, 0.5, 0.5)):
        manager = FilterManager(base.win, base.cam)
        tex1 = Texture()
        tex2 = Texture()
        tex3 = Texture()
        finalquad = manager.renderSceneInto(colortex=tex1)
        # First step - threshold and radial blur
        interquad = manager.renderQuadInto(colortex=tex2)
        interquad.setShader(Shader.load("invert_threshold_r_blur.sha"))
        interquad.setShaderInput("tex1", tex1)
        interquad.setShaderInput("threshold", threshold)        
        # Second step - hardcoded fast gaussian blur. 
        # Not very important. This step can be omitted to improve performance
        # with some minor changes in lens_flare.sha
        interquad2 = manager.renderQuadInto(colortex=tex3)
        interquad2.setShader(Shader.load("gaussian_blur.sha"))
        interquad2.setShaderInput("tex2", tex2)
        # Final - Make lens flare and blend it with the main scene picture
        finalquad.setShader(Shader.load("lens_flare.sha"))
        finalquad.setShaderInput("tex1", tex1)
        finalquad.setShaderInput("tex2", tex2)
        finalquad.setShaderInput("tex3", tex3)       
    
    def update(self, task):
        #update the skybox
        self.spacebox.setPos(base.camera.getPos()) 
        #delta time
        time=globalClock.getDt() 
        #speed 
        self.speeder['text']= '%d' % (self.speed*100)
        #last positon befor impact        
        last_pos=self.shipNonde.getPos()
        #collisions
        base.cTrav.traverse(render)
        self.CollQueue.sortEntries()
        if self.CollQueue.getNumEntries() >0:
            #print "HIT"            
            self.engineOn=False
            self.pLight2.setAttenuation(Point3(0, 0.0, 1)) 
            self.pLight1.setAttenuation(Point3(0, 0.0, 1)) 
            self.glow_back1.hide()
            self.glow_back2.hide() 
            self.glow_front1.hide()
            self.glow_front2.hide() 
            self.sounds['engine'].stop()
            self.sounds['engine2'].stop()
            self.shipNonde.setY(self.shipNonde, +self.speed*0.1)
            self.speed=0
            self.ship.setPos(self.shipNonde.getPos())
            self.cameraNode.setPos(self.shipNonde.getPos())
            self.arrow.setSy(0.1)               
            if self.CollQueue.getEntry(0).getIntoNode().hasTag("end"):
                if self.sounds['at_end'].status() != self.sounds['at_end'].PLAYING:
                    self.sounds['at_end'].play()
                self.ignoreAll()
                self.speed=10
                self.tunel.detachNode()
                self.arrow.hide()
                taskMgr.remove('ticker')
                self.timer['text_fg']=(0,.5,1,1)
                self.keyMap["go"]=0
                self.keyMap["stop"]=0
                self.keyMap["left"]=0
                self.keyMap["right"]=0
                self.engineOn=False 
            else:    
                if self.sounds['ooc'].status() != self.sounds['ooc'].PLAYING:
                    self.sounds['ooc'].play()
            return task.cont
        #engines
        if self.fuel >0:
            direction=0
            if (self.keyMap["go"]!=0):
                direction=2
                #render.setLight(self.backLightNode)
                self.pLight2.setAttenuation(Point3(1, 0.05, 0.02)) 
                self.glow_back1.show()
                self.glow_back2.show()
            if (self.keyMap["go"]==0):
                #render.setLightOff(self.backLightNode)
                self.pLight2.setAttenuation(Point3(0, 0.0, 1)) 
                self.glow_back1.hide()
                self.glow_back2.hide()    
            if (self.keyMap["stop"]!=0):            
                direction=-2  
                #render.setLight(self.frontLightNode)
                self.pLight1.setAttenuation(Point3(1, 0.05, 0.02)) 
                self.glow_front1.show()
                self.glow_front2.show()
            if (self.keyMap["stop"]==0):
                #render.setLightOff(self.frontLightNode)
                self.pLight1.setAttenuation(Point3(0, 0.0, 1)) 
                self.glow_front1.hide()
                self.glow_front2.hide() 
                
            if (self.keyMap["stop"]==1 or self.keyMap["go"]):
                self.engineOn=True
                delta=self.shipNonde.getHpr()-self.ship.getHpr()
                factor=cos(radians(delta[0]))*cos(radians(delta[1]))
                #print factor            
                self.speed=self.speed+(time*direction*factor)            
                if self.speed<0:
                    self.speed=0
                #print self.speed    
                if self.speed==0:
                    self.shipNonde.setHpr(self.ship.getHpr())
                    self.engineOn=False                
            else:
                self.engineOn=False  
            #sound
            if self.engineOn:
                self.fuel-=time*self.fuel_usage
                if direction > 0: #going forward 
                    if self.sounds['engine'].status() != self.sounds['engine'].PLAYING:
                        self.sounds['engine'].play()
                    self.sounds['engine2'].stop()        
                else:
                    if self.sounds['engine2'].status() != self.sounds['engine2'].PLAYING:
                        self.sounds['engine2'].play()
                    self.sounds['engine'].stop()        
            else:
                self.sounds['engine'].stop()
                self.sounds['engine2'].stop()
            #ship rotation        
            if (self.keyMap["left"]!=0):
                self.ship.setH(self.ship.getH() + 140 * time)
            if (self.keyMap["right"]!=0):
                self.ship.setH(self.ship.getH() - 140 * time)
            if (self.keyMap["up"]!=0):
                self.ship.setP(self.ship.getP() - 140 * time)
            if (self.keyMap["down"]!=0):
                self.ship.setP(self.ship.getP() + 140 * time)    
            
        #camera rotation
        if self.lockCamera:
            self.cameraNode.setHpr(self.ship.getHpr())
        else:
            if base.mouseWatcherNode.hasMouse():            
                mpos = base.mouseWatcherNode.getMouse()                              
                self.cameraNode.setH(self.cameraNode.getH()-mpos.getX()*time*90)   
                self.cameraNode.setP(self.cameraNode.getP()-mpos.getY()*time*90)                      
        #update the arrow/ship
        self.shipNonde.setY(self.shipNonde, -self.speed*0.02)
        self.ship.setPos(self.shipNonde.getPos())
        self.cameraNode.setPos(self.shipNonde.getPos())
        self.arrow.setSy((self.speed/2)+0.1) #the +0.1 is to keep the scale from being 0. 0 scale is wrong. 
        return task.cont
        
    def shipRotation(self, task):
        if self.engineOn:
            origHpr = self.shipNonde.getHpr()
            targetHpr = self.ship.getHpr()
            # Make the rotation go the shortest way around.
            origHpr = VBase3(fitSrcAngle2Dest(origHpr[0], targetHpr[0]),
                             fitSrcAngle2Dest(origHpr[1], targetHpr[1]),
                             fitSrcAngle2Dest(origHpr[2], targetHpr[2]))

            # How far do we have to go from here?
            delta = max(abs(targetHpr[0] - origHpr[0]),
                        abs(targetHpr[1] - origHpr[1]),
                        abs(targetHpr[2] - origHpr[2]))
            if delta == 0:
                # We're already looking at the target.
                return task.cont
            # Figure out how far we should rotate in this frame, based on the
            # distance to go, and the speed we should move each frame.
            t = globalClock.getDt() * (300/self.speed) / delta
            # If we reach the target, stop there.
            t = min(t, 1.0)
            newHpr = origHpr + (targetHpr - origHpr) * t
            self.shipNonde.setHpr(newHpr)
        return task.cont
        
#loading screen
base.setBackgroundColor(0, 0, 0)
winX = (wp.getXSize()-640)/2
winY = (wp.getYSize()-480)/2
load_screen = DirectFrame(frameSize=(-1024, 0, 0, 1024),
                                    frameColor=(1, 1, 1, 1),
                                    frameTexture='tex/loading.png',
                                    parent=pixel2d)
load_screen.setPos(1024+winX,0,-1024-winY)  
for i in range(5):
    base.graphicsEngine.renderFrame()                                      
w = World()    
run()               